import numpy as np
from numpy.linalg import lstsq, inv
from numpy.random import normal, gamma
from numpy.random import multivariate_normal as mvnormal

def regGibbs(y,X,nit):

    # remove NaNs
    knan = ~np.isnan(y) & ~np.isnan(np.mean(X,1))
    y = y[knan]
    X = X[knan,:]
    nt = y.shape[0]
    nx = X.shape[1]

    # ---------- compute Bayesian regression with autocorrelated errors ----------
    beta = np.zeros((nx,nit)) # preallocate array
    se = np.zeros((nit,))
    rho = np.zeros((nit,))

    # initialize parameters by running OLS
    bols = lstsq(X,y,rcond=None)[0]
    res = y - np.dot(X,bols)
    SSR = np.dot(res.T,res) # sum of squares of residuals
    dof = nt - X.shape[1] # degrees of freedom
    covb = SSR/dof*inv(np.dot(X.T,X)) # OLS covariance matrix of coefficients
    beta[:,0] = bols
    se[0] = np.var(res)
    rho[0] = np.corrcoef(res[0:-1],res[1:])[1,0]

    # sample parameters using Gibbs sampling
    mb0 = bols # mean of the normal prior on beta
    icovb0 = np.diag(1/(np.diag(covb)*10)) # inverse covariance matrix of the normal prior on beta
    varRho0 = 0.2 # variance of the normal prior on rho
    for i in range(1,nit):

        # sample "rho" from the conditional distribution p(rho|y,beta^(i-1),se^(i-1))
        r = y - np.dot(X,beta[:,i-1])
        v = 1/(1/varRho0 + 1/se[i-1]*np.dot(r[0:-1].T,r[0:-1]))
        mu = v*(rho[0]/varRho0 + 1/se[i-1]*np.dot(r[0:-1].T,r[1:]))
        rho[i] = 2
        while np.abs(rho[i]) > 1: # rho must lie in the range -1<rho<1
            rho[i] = normal(mu,np.sqrt(v))

        # sample "beta" from the conditional distribution p(beta|y,rho^(i),se^(i-1))
        ys = y[1:] - rho[i]*y[0:-1]
        Xs = X[1:,:] - rho[i]*X[0:-1,:]
        covb = inv(icovb0 + 1/se[i-1]*np.dot(Xs.T,Xs))
        covb = (covb + covb.T)/2.
        mu = np.dot(covb,np.dot(icovb0,mb0) + 1/se[i-1]*np.dot(Xs.T,ys))
        beta[:,i] = mvnormal(mu,covb)

        # sample "se" from the conditional distribution p(se|y,rho^(i),beta^(i))
        db = np.dot((ys - np.dot(Xs,beta[:,i])).T,ys - np.dot(Xs,beta[:,i]))
        se[i] = 1/gamma(0.01+(nt-1)/2,1/(0.01 + db/2.))

    return beta,se,rho,bols






        
        




 
