import numpy as np
from datetime import datetime, timedelta
import regGibbs
from scipy.stats import circmean, circstd

def seasonalCycle(y,t,nit = 10000):
    wa = 2*np.pi/365.24 # angular velocity annual cycle
    wsa = 2*np.pi/182.62 # angular velocity semi-annual cycle

    # arrange time vector
    #t0 = datetime.utcfromtimestamp(t[0])
    t0 = datetime(1970,1,1) + timedelta(seconds=t[0])
    yr0 = t0.year
    epoch = datetime.utcfromtimestamp(0) # 1970-01-01 00:00:00
    t0 = (datetime(yr0,1,1,0,0) - epoch).total_seconds() 
    t = (t - t0)/(24.*3600) # in days

    # construct regressors vector
    nt = y.shape[0]
    X = np.array([np.ones((nt,)), t, np.cos(wa*t), np.sin(wa*t), np.cos(wsa*t), np.sin(wsa*t)]).T
    nx = X.shape[1]

    # compute regression
    beta,se,rho,bols = regGibbs.regGibbs(y,X,nit)
    burn = int(np.asscalar(np.fix(0.1*nit))) # burn-in period (10% of the total number of iterations)
    beta = beta[:,burn:] 
    rho = rho[burn:]
    se = se[burn:]

    Aa = np.sqrt(beta[2,:]*beta[2,:] + beta[3,:]*beta[3,:])
    seAa = np.std(Aa)
    Aa = np.mean(Aa)
    phia = np.arctan2(beta[3,:],beta[2,:])
    sePa = circstd(phia,low=-np.pi,high=np.pi)
    phia = circmean(phia,low=-np.pi,high=np.pi)
    phia = np.mod(phia + 2*np.pi,2*np.pi) # convert from (-pi,pi) to (0,2*pi)
    phia = phia*365/(2*np.pi) #convert to days
    sePa = sePa*365/(2*np.pi)

    Asa = np.sqrt(beta[4,:]*beta[4,:] + beta[5,:]*beta[5,:])
    seAsa = np.std(Asa)
    Asa = np.mean(Asa)
    phisa = np.arctan2(beta[5,:],beta[4,:])
    psa = phisa
    sePsa = circstd(phisa,low=-np.pi,high=np.pi) # compute circular mean and standard deviation of posterior
    phisa = circmean(phisa,low=-np.pi,high=np.pi)
    phisa = np.mod(phisa + 2*np.pi,2*np.pi) # convert from (-pi,pi) to (0,2*pi)
    phisa = phisa*182.5/(2*np.pi) #convert to days
    sePsa = sePsa*182.5/(2*np.pi)
    
    seTr = np.std(beta[1,:])*365.24*1000 #convert to mm/yr
    tr = np.mean(beta[1,:])*365.24*1000 #convert to mm/yr
    
    # compute residual
    beta = np.mean(beta,1)
    res = np.zeros((nt,))*np.NaN
    knan = ~np.isnan(t)
    res[knan] = y[knan] - np.dot(X[knan,:],beta)

    results = {"amp_an":Aa,"amp_an_se":seAa,"phase_an":phia,"phase_an_se":sePa,"amp_semian":Asa,
               "amp_semian_se":seAsa,"phase_semian":phisa,"phase_semian_se":sePsa,"residual":res,
               "trend":tr,"trend_se":seTr}
    
    return results








        
        




 
